package ch.dlcm.solidify;

import java.io.FileNotFoundException;
import java.io.IOException;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;

@Mojo(name = "clean-json", defaultPhase = LifecyclePhase.PREPARE_PACKAGE)
public class OpenApiCleanerMojo extends AbstractMojo {



  @Parameter(property = "clean.inputFilePath", required = true)
  private String inputFilePath;

  @Parameter(property = "clean.inputExtraDocumentationFilePath", required = true)
  private String inputExtraDocumentationFilePath;

  @Parameter(property = "clean.outputFilePath", required = true)
  private String outputFilePath;

  @Parameter(property = "clean.servletContextPath", required = true)
  private java.lang.String servletContextPath;

  @Parameter(property = "clean.skip", required = false, defaultValue = "false")
  private boolean skip;

  public void execute() throws MojoExecutionException {
    if(skip) {
      getLog().info("Skipping clean of openAPI json file");
      return;
    }
    JsonCleaner jsonCleaner = new JsonCleaner();
    try {
      getLog().info(getFileReadLogMessage());
      jsonCleaner.load(inputFilePath, inputExtraDocumentationFilePath);
      getLog().info("Process JSON Cleanup");
      jsonCleaner.run(servletContextPath);
      getLog().info("Write file in " + outputFilePath);
      jsonCleaner.generate(outputFilePath);
      getLog().info("Successfully processed JSON Object in " + outputFilePath);
    } catch (FileNotFoundException e) {
      e.printStackTrace();
      getLog().info("File not found: " + e.getMessage());
      throw new MojoExecutionException(e.getMessage(), e);
    } catch (IOException e) {
      e.printStackTrace();
      getLog().info("IO Exception: " + e.getMessage());
      throw new MojoExecutionException(e.getMessage(), e);
    }
  }

  private String getFileReadLogMessage() {
    StringBuilder messageReadFileBuilder = new StringBuilder();
    messageReadFileBuilder.append("Read openapi file : " + inputFilePath);
    if (inputExtraDocumentationFilePath == null) {
      messageReadFileBuilder.append(" and no extra documentation provided");
    } else {

      messageReadFileBuilder
              .append(" and extra documentation file : " + inputExtraDocumentationFilePath);
    }
    return messageReadFileBuilder.toString();
  }
}
