package ch.dlcm.solidify.process;

import java.util.Iterator;

import org.json.JSONObject;

/**
 * Add suffix (/get, /post, /get-by-id, /delete-by-id, /patch-by-id) to operation id for each verb
 * (GET, POST, DELETE, PATCH)
 */
public class PathAddSuffixOnOperationIdProcess extends BaseProcess {
  public PathAddSuffixOnOperationIdProcess() {
  }

  public void process(JSONObject json) {
    for (Iterator<String> i = this.getPathsNode(json).keys(); i.hasNext();) {
      String pathName = i.next();
      JSONObject path = this.getPathsNode(json).getJSONObject(pathName);

      boolean isWithId = determinePathWithId(pathName);
      iterateOverPathVerb(path, isWithId);
    }
  }

  private boolean determinePathWithId(String pathName) {
    return pathName.endsWith(RES_ID_URL_PARAM);
  }

  private void iterateOverPathVerb(JSONObject path, boolean isWithId) {
    for (Iterator<String> i = path.keys(); i.hasNext();) {
      String verbName = i.next();
      JSONObject verb = path.getJSONObject(verbName);

      String suffix = getSuffix(verbName, isWithId);
      applySuffixToOperationId(verb, suffix);
    }
  }

  private String getSuffix(String verbName, boolean isWithId) {
    return URL_SEPARATOR + verbName + (isWithId ? BY_ID_SUFFIX : EMPTY);
  }

  private void applySuffixToOperationId(JSONObject verb, String suffix) {
    String operationId = verb.getString(OPERATION_ID_KEY);
    verb.put(OPERATION_ID_KEY, operationId + suffix);
  }
}
