package ch.dlcm.solidify.process;

import java.util.Iterator;

import org.json.JSONObject;

import ch.dlcm.solidify.model.QueryParametersObject;

/**
 * Add to existing path with POST verb the other verb GET that return collection (reference the
 * schema collection and add query parameters)
 */
public class PathGetCollectionProcess extends BaseProcess {

  public PathGetCollectionProcess() {
  }

  public void process(JSONObject json) {
    for (Iterator<String> i = this.getPathsNode(json).keys(); i.hasNext();) {
      String pathName = i.next();
      JSONObject path = this.getPathsNode(json).getJSONObject(pathName);

      // Check if resource collection web service only
      // not application, not module, not resource-by-id
      if (this.countCharacter(pathName, '/') != 2) {
        continue;
      }

      if (!this.isReadOnlyResource(path)) {
        JSONObject postPath = (JSONObject) path.query(QUERY_JSON_PREFIX + POST_PATH);
        // Update response to 200
        postPath = updateResponse(postPath, "201", "200");
        // Create collection get
        createGetPath(path, postPath);
      } else {
        JSONObject getPath = (JSONObject) path.query(QUERY_JSON_PREFIX + GET_PATH);
        // Create collection get
        createGetPath(path, getPath);
      }
    }
  }

  private void createGetPath(JSONObject path, JSONObject postPath) {
    JSONObject getPath = generateGetPathFromPostPath(postPath);
    path.put(GET_PATH, getPath);
  }

  private JSONObject generateGetPathFromPostPath(JSONObject postPath) {
    JSONObject getPath = deepCopy(postPath);
    getPath.remove(REQUEST_BODY_KEY);
    updateSchemaRefToCollection(getPath);
    addQueryParameters(getPath);
    return getPath;
  }

  private void updateSchemaRefToCollection(JSONObject getPath) {
    JSONObject schema = getSchemaNode(getPath);
    schema.put(REF_KEY, REF_SCHEMAS_VALUE + COLLECTION_NAME);
  }

  private JSONObject getSchemaNode(JSONObject getPath) {
    JSONObject responses = getPath.getJSONObject(RESPONSES_KEY);
    JSONObject response200 = getFirstChild(responses);
    JSONObject content = response200.getJSONObject(CONTENT_KEY);
    JSONObject applicationHalJson = getFirstChild(content);
    return applicationHalJson.getJSONObject(SCHEMA_KEY);
  }

  private void addQueryParameters(JSONObject getPath) {
    JSONObject queryParameters = convertObjectAsJsonObject(new QueryParametersObject());
    getPath.put(PARAMETERS_KEY, queryParameters.getJSONArray(PARAMETERS_KEY));
  }
}
