package ch.dlcm.solidify.process;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONObject;

import ch.dlcm.solidify.model.ParametersObject;

/**
 * Add new path with id (for GET, PATCH, DELETE)
 */
public class PathWithIdCreationProcess extends BaseProcess {
  private JSONArray paramatersJsonArray;

  public PathWithIdCreationProcess() {
    paramatersJsonArray = getParametersJsonArray();
  }

  public void process(JSONObject json) {
    Map<String, JSONObject> pathsWithId = new HashMap<>();
    iterateOverPostPaths(json, pathsWithId);
    json.put(PATHS_KEY, mergeJsonObject(this.getPathsNode(json), new JSONObject(pathsWithId)));
  }

  private void iterateOverPostPaths(JSONObject json, Map<String, JSONObject> pathsWithId) {
    for (Iterator<String> i = this.getPathsNode(json).keys(); i.hasNext();) {
      String pathName = i.next();
      JSONObject path = this.getPathsNode(json).getJSONObject(pathName);

      // Check if resource web service, not application, not module
      if (this.countCharacter(pathName, '/') == 1) {
        continue;
      }

      boolean readOnlyResource = this.isReadOnlyResource(path);

      // Get http verb template
      JSONObject httpPath = this.getHttpVerb(path);
      if (httpPath == null) {
        continue;
      }

      // Update response to 200
      if (!readOnlyResource) {
        httpPath = updateResponse(httpPath, "201", "200");
      }

      // Add new web services (GET by ID, DELETE, PATH)
      addPathWithId(pathsWithId, pathName, httpPath, readOnlyResource);
    }
  }

  private JSONObject getHttpVerb(JSONObject path) {
    JSONObject post = (JSONObject) path.query(QUERY_JSON_PREFIX + POST_PATH);
    if (post != null) {
      return post;
    }
    return (JSONObject) path.query(QUERY_JSON_PREFIX + GET_PATH);
  }

  private void addPathWithId(Map<String, JSONObject> pathsWithId, String pathName,
          JSONObject postPath, boolean readOnlyResource) {
    JSONObject pathsWithIdContent = new JSONObject();
    createVerbs(postPath, pathsWithIdContent, readOnlyResource);
    pathsWithId.put(pathName + URL_SEPARATOR + RES_ID_URL_PARAM, pathsWithIdContent);
  }

  private void createVerbs(JSONObject httpPath, JSONObject pathsWithIdContent,
          boolean readOnlyResource) {
    createGetPath(pathsWithIdContent, deepCopy(httpPath));
    if (!readOnlyResource) {
      createPatchPath(pathsWithIdContent, deepCopy(httpPath));
      createDeletePath(pathsWithIdContent, deepCopy(httpPath));
    }
  }

  private JSONArray getParametersJsonArray() {
    JSONArray jsonArray = new JSONArray();
    jsonArray.put(getParameterJsonObject());
    return jsonArray;
  }

  private JSONObject getParameterJsonObject() {
    return new JSONObject(getParametersJson());
  }

  private String getParametersJson() {
    return this.toJson(new ParametersObject());
  }

  private void createPatchPath(JSONObject pathsWithIdContent, JSONObject patchPath) {
    patchPath.put(PARAMETERS_KEY, paramatersJsonArray);
    pathsWithIdContent.put(PATCH_PATH, patchPath);
  }

  private void createGetPath(JSONObject pathsWithIdContent, JSONObject getPath) {
    getPath.remove(REQUEST_BODY_KEY);
    getPath.put(PARAMETERS_KEY, paramatersJsonArray);
    pathsWithIdContent.put(GET_PATH, getPath);
  }

  private void createDeletePath(JSONObject pathsWithIdContent, JSONObject deletePath) {
    deletePath.remove(REQUEST_BODY_KEY);
    deletePath.put(PARAMETERS_KEY, paramatersJsonArray);
    pathsWithIdContent.put(DELETE_PATH, deletePath);
  }
}
