package ch.dlcm.solidify.process;

import java.util.Iterator;

import org.json.JSONObject;

import ch.dlcm.solidify.model.ChangeInfoObject;

/**
 * Force type of lastUpdate and creation attribute to ChangeInfo
 */
public class SchemaChangeInfoProcess extends BaseProcess {
  private JSONObject changeInfoJsonObject;

  public SchemaChangeInfoProcess() {
    changeInfoJsonObject = getChangeInfoJsonObject();
  }

  public void process(JSONObject json) {
    // Add creation & lastUpdate field
    iterateOverSchema(json);
    // Add changInfo info in schemas
    addChangeInfo(json);
  }

  private JSONObject getChangeInfoJsonObject() {
    return new JSONObject(getChangeInfoJson());
  }

  private String getChangeInfoJson() {
    return this.toJson(new ChangeInfoObject());
  }

  private void iterateOverSchema(JSONObject json) {
    for (Iterator<String> i = this.getSchemasNode(json).keys(); i.hasNext();) {
      String schemaName = i.next();
      JSONObject schema = this.getSchemasNode(json).getJSONObject(schemaName);
      JSONObject propertiesNode = (JSONObject) schema.query(QUERY_JSON_PREFIX + PROPERTIES_KEY);
      if (propertiesNode == null)
        continue;
      setChangeInfoType(propertiesNode, ATTRIBUTE_LAST_UPDATE);
      setChangeInfoType(propertiesNode, ATTRIBUTE_CREATION);
    }
  }

  private void setChangeInfoType(JSONObject propertiesNode, String key) {
    JSONObject node = (JSONObject) propertiesNode.query(QUERY_JSON_PREFIX + key);
    if (node == null)
      return;
    node.put(TYPE_KEY, CHANGE_INFO_NAME);
  }

  private void addChangeInfo(JSONObject json) {
    this.getSchemasNode(json).put(CHANGE_INFO_NAME, changeInfoJsonObject);
  }
}
