package ch.dlcm.solidify.process;

import java.util.List;

import org.json.JSONObject;

import ch.dlcm.solidify.model.OpenApiExtraDocumentation;
import ch.dlcm.solidify.model.OpenApiExtraDocumentationProperties;

/**
 * Force type of all array into string
 */
public class SchemaExtraDocumentationProcess extends BaseProcess {

  private List<OpenApiExtraDocumentation> listOpenApiExtraDocumentation;

  public SchemaExtraDocumentationProcess(String jsonExtraDocumentation) {
    this.listOpenApiExtraDocumentation = this.loadOpenApiExtraDocumentation(jsonExtraDocumentation);
  }

  public void process(JSONObject json) {
    // Update schemas with extra info
    for (OpenApiExtraDocumentation extraDoc : listOpenApiExtraDocumentation) {
      if (!extraDoc.getProperties().isEmpty()) {
        JSONObject schemas = this.getSchemasNode(json);
        JSONObject schema = schemas.getJSONObject(extraDoc.getSchema());
        iterateOverExtraDocumentationProperties(schema, extraDoc.getProperties());
      }
    }
  }

  private void iterateOverExtraDocumentationProperties(JSONObject schema,
          List<OpenApiExtraDocumentationProperties> listExtraDocProperties) {
    for (OpenApiExtraDocumentationProperties extraDocProperties : listExtraDocProperties) {
      if (extraDocProperties.getFieldName() == null && extraDocProperties.getEnumsValue() == null)
        continue;

      JSONObject propertiesNode = schema.getJSONObject(PROPERTIES_KEY);
      JSONObject propertyNode = (JSONObject) propertiesNode
              .query(QUERY_JSON_PREFIX + extraDocProperties.getFieldName());

      if (propertyNode == null)
        continue;

      setFormatAndType(extraDocProperties, propertyNode);
      setEnumValues(extraDocProperties, propertyNode);
      setItem(extraDocProperties, propertyNode);
    }
  }

  private void setItem(OpenApiExtraDocumentationProperties extraDocProperties,
          JSONObject propertyNode) {
    String item = extraDocProperties.getItem();
    if (item == null)
      return;
    propertyNode.getJSONObject(ITEMS_KEY).put(TYPE_KEY, item);
  }

  private void setEnumValues(OpenApiExtraDocumentationProperties extraDocProperties,
          JSONObject propertyNode) {
    List<String> enumsValue = extraDocProperties.getEnumsValue();
    if (enumsValue == null)
      return;
    propertyNode.put(ENUM_KEY, convertObjectAsJsonArray(enumsValue));
  }

  private void setFormatAndType(OpenApiExtraDocumentationProperties extraDocProperties,
          JSONObject propertyNode) {
    String format = extraDocProperties.getFormat();
    if (format == null)
      return;
    propertyNode.put(FORMAT_KEY, format);
    updateTypeIfNecessary(propertyNode, format);
  }

  private void updateTypeIfNecessary(JSONObject propertyNode, String format) {
    String type = propertyNode.getString(TYPE_KEY);
    String newType = type;
    switch (format) {
      case FORMAT_INTEGER_INT32:
      case FORMAT_INTEGER_INT64:
        newType = INTEGER;
        break;
    }
    boolean isDifferentType = !newType.equals(type);
    if (isDifferentType)
      propertyNode.put(TYPE_KEY, type);
  }
}
