package ch.dlcm.solidify;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.util.Iterator;

import org.json.JSONObject;

/**
 * Assert after execution of plugin
 */
public class AssertPathsProcessImpl extends AssertBaseProcess {
  public AssertPathsProcessImpl(JSONObject jsonBefore) {
    super(jsonBefore);
  }

  public void process(JSONObject jsonAfter) {
    JSONObject pathsJson = this.getPathsNode(jsonBefore);
    int numberOfPost = countResourcePathsChild(pathsJson, "post");
    int numberOfGet = countResourcePathsChild(pathsJson, "get");

    int numberOfPostVerbsCreated = 4; // GET, GET-BY-ID, PATCH, DELETE
    int numberOfGetVerbsCreated = 1; // GET-BY-ID
    int expectedNumberOfVerb = pathsJson.length() + numberOfPost * numberOfPostVerbsCreated
            + numberOfGet * numberOfGetVerbsCreated;

    int currentNumberOfVerb = countPathsChild(this.getPathsNode(jsonAfter));
    assertEquals(expectedNumberOfVerb, currentNumberOfVerb);
  }

  private int countResourcePathsChild(JSONObject pathNode, String httpVerb) {
    int cpt = 0;
    for (Iterator<String> i = pathNode.keys(); i.hasNext();) {
      String pathName = i.next();

      // Check if resource web service, not application, not module
      // ex: /dlcm/<module>
      if (this.countCharacter(pathName, '/') < 3) {
        continue;
      }

      // Check http verb exist
      if (pathNode.getJSONObject(pathName).isNull(httpVerb)) {
        continue;
      }

      cpt++;
    }
    return cpt;
  }

  private int countPathsChild(JSONObject pathNode) {
    int cpt = 0;
    for (Iterator<String> i = pathNode.keys(); i.hasNext();) {
      String pathName = i.next();
      JSONObject path = pathNode.getJSONObject(pathName);
      cpt += path.length();
    }
    return cpt;
  }

}
